<?php

/**
 * Suppress "error - 0 - No summary was found for this file" on phpdoc generation
 *
 * @package WPDataAccess\Simple_Form
 */

namespace WPDataAccess\Simple_Form {

	/**
	 * Class WPDA_Simple_Form_Item_DateTime
	 *
	 * Adds a date/time picker to fields of type: date, datetime, timestamp and time
	 *
	 * @author  Peter Schulz
	 * @since   2.5.1
	 */
	class WPDA_Simple_Form_Item_DateTime extends WPDA_Simple_Form_Item {

		protected $date_format = 'Y-m-d';
		protected $date_picker = 'true';
		protected $time_picker = 'false';

		/**
		 * WPDA_Simple_Form_Item_DateTime constructor.
		 *
		 * @param array $args
		 */
		public function __construct( $args = [] ) {
			parent::__construct( $args );

			if ( 'timestamp' === $this->column_type || 'datetime' === $this->column_type ) {
				$this->date_format = 'Y-m-d H:i:s';
				$this->time_picker = 'true';
				$this->item_placeholder = 'yyyy-mm-dd hh:mi:ss';
			} elseif ('time' === $this->column_type) {
				$this->date_format = 'H:i:s';
				$this->date_picker = 'false';
				$this->time_picker = 'true';
				$this->item_placeholder = 'hh:mi:ss';
			} else {
				$this->item_placeholder = 'yyyy-mm-dd';
			}
		}

		/**
		 * Overwrite method show_item: add date/time picker
		 */
		public function show_item() {
			parent::show_item();

			?>
			<script language="JavaScript">
				jQuery(document).ready(function() {
					jQuery.datetimepicker.setLocale('en');
					jQuery('#<?php echo esc_attr( $this->item_name ); ?>').datetimepicker({
						format: '<?php echo $this->date_format; ?>',
						datepicker: <?php echo $this->date_picker; ?>,
						timepicker: <?php echo $this->time_picker; ?>
					});
				});
			</script>
			<?php
		}

	}

}