<?php

/**
 * Suppress "error - 0 - No summary was found for this file" on phpdoc generation
 *
 * @package WPDataProjects\Project
 */

namespace WPDataProjects\Project {

	use WPDataAccess\Data_Dictionary\WPDA_List_Columns_Cache;
	use WPDataAccess\Design_Table\WPDA_Design_Table_Model;
	use WPDataAccess\Utilities\WPDA_Message_Box;

	/**
	 * Class WPDP_Project_Design_Table_Model extends WPDA_Design_Table_Model
	 *
	 * @see WPDA_Design_Table_Model
	 *
	 * @author  Peter Schulz
	 * @since   2.0.0
	 */
	class WPDP_Project_Design_Table_Model extends WPDA_Design_Table_Model {

		/**
		 * Cached table options
		 *
		 * @var null|array
		 */
		protected static $cache_table_options = null;

		/**
		 * Requested action
		 *
		 * @var string
		 */
		protected $action2;

		/**
		 * WPDP_Project_Design_Table_Model constructor
		 *
		 * Get action2 arguments
		 */
		public function __construct() {
			parent::__construct();

			$this->wpda_table_name_original = $this->wpda_table_name;
			if ( isset( $_REQUEST['action2'] ) ) {
				$this->action2 = sanitize_text_field( wp_unslash( $_REQUEST['action2'] ) );
			}
		}

		/**
		 * Overwrite method validate to add table options validation
		 *
		 * @return array
		 */
		public function validate() {
			$structure_messages = parent::validate();

			if ( isset( $this->wpda_table_design->tableform_column_options ) ) {
				if ( count( $this->wpda_table_design->table ) !== count( $this->wpda_table_design->tableform_column_options ) ) {
					$structure_messages[] = [ 'ERR', 'Invalid structure [run reconcile]' ];
				}
			}

			return $structure_messages;
		}

		/**
		 * Overwrite method prepare_update to prepare table options
		 *
		 */
		public function prepare_update() {
			parent::prepare_update();

			switch ( $this->action2 ) {
				case 'relation':
					$this->prepare_update_relation();
					break;
				case 'listtable':
					$this->prepare_update_listtable();
					break;
				case 'tableform':
					$this->prepare_update_tableform();
					break;
				case 'tableinfo':
					$this->prepare_update_tableinfo();
			}
		}

		/**
		 * Set relationships to prepare update
		 */
		protected function prepare_update_relation() {
			unset( $this->wpda_table_design->relationships );
			if ( isset( $_REQUEST['row_num'] ) ) {
				$no_columns = count( $_REQUEST['row_num'] );
				if (
					isset( $_REQUEST['relation_type'] ) &&
					$no_columns === count( $_REQUEST['relation_type'] ) &&
					isset( $_REQUEST['source_column_name'] ) &&
					$no_columns === count( $_REQUEST['source_column_name'] ) &&
					isset( $_REQUEST['target_table_name'] ) &&
					$no_columns === count( $_REQUEST['target_table_name'] ) &&
					isset( $_REQUEST['target_column_name'] ) &&
					$no_columns === count( $_REQUEST['target_column_name'] )
				) {
					for ( $i = 0; $i < $no_columns; $i ++ ) {
						$relation_type      = sanitize_text_field( wp_unslash( $_REQUEST['relation_type'][ $i ] ) );
						$source_column_name = sanitize_text_field( wp_unslash( $_REQUEST['source_column_name'][ $i ] ) );
						$target_table_name  = sanitize_text_field( wp_unslash( $_REQUEST['target_table_name'][ $i ] ) );
						$target_column_name = sanitize_text_field( wp_unslash( $_REQUEST['target_column_name'][ $i ] ) );

						if ( 'nm' === $relation_type ) {
							$relation_table_name = sanitize_text_field( wp_unslash( $_REQUEST[ 'relation_table_name_' . $i ] ) );
							if ( trim( $relation_table_name ) === '' ) {
								$msg = new WPDA_Message_Box(
									[
										'message_text'           => __( 'Invalid array [missing required fields]', 'wp-data-access' ),
										'message_type'           => 'error',
										'message_is_dismissible' => false,
									]
								);
								$msg->box();

								return;
							}
						}

						if (
							trim( $relation_type ) !== '' &&
							trim( $source_column_name ) !== '' &&
							trim( $target_table_name ) !== '' &&
							trim( $target_column_name ) !== ''
						) {
							$source_column_name_array = [];
							$target_column_name_array = [];

							array_push( $source_column_name_array, $source_column_name );
							array_push( $target_column_name_array, $target_column_name );

							if ( isset( $_REQUEST['num_source_column_name'][ $i ] ) ) {
								$num_source_column_name = sanitize_text_field( wp_unslash( $_REQUEST['num_source_column_name'][ $i ] ) );
								if ( is_numeric( $num_source_column_name ) ) {
									for ( $j = 1; $j <= $num_source_column_name; $j ++ ) {
										if (
											isset( $_REQUEST[ 'source_column_name_' . $i . '_' . $j ] ) &&
											isset( $_REQUEST[ 'target_column_name_' . $i . '_' . $j ] )
										) {
											array_push( $source_column_name_array, sanitize_text_field( wp_unslash( $_REQUEST[ 'source_column_name_' . $i . '_' . $j ] ) ) );
											array_push( $target_column_name_array, sanitize_text_field( wp_unslash( $_REQUEST[ 'target_column_name_' . $i . '_' . $j ] ) ) );
										}
									}
								}
							}

							$this->wpda_table_design->relationships[ $i ]['relation_type']      = $relation_type;
							$this->wpda_table_design->relationships[ $i ]['source_column_name'] = $source_column_name_array;
							$this->wpda_table_design->relationships[ $i ]['target_table_name']  = $target_table_name;
							$this->wpda_table_design->relationships[ $i ]['target_column_name'] = $target_column_name_array;
							if ( 'nm' === $relation_type ) {
								$this->wpda_table_design->relationships[ $i ]['relation_table_name'] = $relation_table_name;
							}
						}
					}
				} else {
					$msg = new WPDA_Message_Box(
						[
							'message_text'           => __( 'Invalid array [missing required fields]', 'wp-data-access' ),
							'message_type'           => 'error',
							'message_is_dismissible' => false,
						]
					);
					$msg->box();
				}
			}
		}

		/**
		 * Set table options for list table to prepare update
		 */
		protected function prepare_update_listtable() {
			$column_options = $this->get_column_options_from_request();
			if ( null !== $column_options ) {
				$this->wpda_table_design->listtable_column_options = $column_options;
			}
		}

		/**
		 * Set table options for data entry form to prepare update
		 */
		protected function prepare_update_tableform() {
			$column_options = $this->get_column_options_from_request();
			if ( null !== $column_options ) {
				$this->wpda_table_design->tableform_column_options = $column_options;
			}
		}

		/**
		 * Set table options for tab label to prepare update
		 */
		protected function prepare_update_tableinfo() {
			if ( isset( $_REQUEST['tab_label'] ) ) {
				$this->wpda_table_design->tableinfo = [
					'tab_label' => sanitize_text_field( wp_unslash( $_REQUEST['tab_label'] ) ),
				];
			}
		}

		/**
		 * Get table column options
		 *
		 * @return array|null
		 */
		protected function get_column_options_from_request() {
			if ( isset( $_REQUEST['list_item_name'] ) ) {
				$tableform_column_options = [];
				$i                        = 0;
				foreach ( $_REQUEST['list_item_name'] as $column_name ) {
					$tableform_column_options[] = [
						'column_name' => $column_name,
						'label'       => isset( $_REQUEST[ $column_name ] ) ?
							sanitize_text_field( wp_unslash( $_REQUEST[ $column_name ] ) ) :
							ucfirst( str_replace( '_', ' ', $column_name ) ),
						'less'        => isset( $_REQUEST["{$column_name}_less"] ) ? 'on' : 'off',
						'show'        => isset( $_REQUEST["{$column_name}_show"] ) ? 'on' : 'off',
						'lookup'      => isset( $_REQUEST["{$column_name}_lookup"] ) ?
							sanitize_text_field( wp_unslash( $_REQUEST["{$column_name}_lookup"] ) ) :
							false,
						'item_type'   => isset( $_REQUEST["{$column_name}_item_type"] ) ?
							sanitize_text_field( wp_unslash( $_REQUEST["{$column_name}_item_type"] ) ) : 'default',
					];
					if ( isset( $_REQUEST["{$column_name}_default"] ) && '' !== $_REQUEST["{$column_name}_default"] ) {
						$tableform_column_options[ $i ]['default'] =
							sanitize_text_field( wp_unslash( $_REQUEST["{$column_name}_default"] ) );
					}
					$i ++;
				}

				return $tableform_column_options;
			} else {
				return null;
			}
		}

		/**
		 * Reconcile table
		 *
		 * @param object $table_structure Table structure
		 * @param string $param_keep_options Possible values: 'on' and 'off'
		 *
		 * @return mixed Result of SQL update
		 */
		public function reconcile(
			$table_structure,
			$param_keep_options
		) {
			if ( 1 === $this->query( $this->wpda_table_name_original === $this->wpda_table_name ? null : $this->wpda_table_name_original ) ) {
				// Table definition.
				$this->wpda_table_design->design_mode = $table_structure['design_mode'];
				$this->wpda_table_design->engine      = $table_structure['engine'];
				$this->wpda_table_design->collation   = $table_structure['collation'];
				// Table columns.
				$this->wpda_table_design->table = $table_structure['table'];
				// Table indexes.
				$this->wpda_table_design->indexes = $table_structure['indexes'];

				if ( 'on' !== $param_keep_options ) {
					// Clear arrays.
					$this->wpda_table_design->listtable_column_options = [];
					$this->wpda_table_design->tableform_column_options = [];
					$this->wpda_table_design->tableinfo                = [];
				} else {
					// Remove non existing columns from arrays.
					$new_listtable_column_options = [];
					foreach ( $this->wpda_table_design->listtable_column_options as $listtable_column_option ) {
						$column_found = false;
						foreach ( $this->wpda_table_design->table as $table_column ) {
							if ( $table_column->column_name === $listtable_column_option->column_name ) {
								$column_found = true;
							}
						}
						if ( $column_found ) {
							// Add only column to array that were found in the table definition.
							array_push( $new_listtable_column_options, $listtable_column_option );
						}
					}
					$this->wpda_table_design->listtable_column_options = $new_listtable_column_options;
					$new_tableform_column_options                      = [];
					foreach ( $this->wpda_table_design->tableform_column_options as $tableform_column_option ) {
						$column_found = false;
						foreach ( $this->wpda_table_design->table as $table_column ) {
							if ( $table_column->column_name === $tableform_column_option->column_name ) {
								$column_found = true;
							}
						}
						if ( $column_found ) {
							// Add only column to array that were found in the table definition.
							array_push( $new_tableform_column_options, $tableform_column_option );
						}
					}
					$this->wpda_table_design->tableform_column_options = $new_tableform_column_options;
				}

				$wpda_list_columns    = WPDA_List_Columns_Cache::get_list_columns( '', $this->wpda_table_name );
				$table_column_headers = $wpda_list_columns->get_table_column_headers();

				foreach ( $this->wpda_table_design->table as $column ) {
					if ( 'on' !== $param_keep_options ) {
						// Every column must be added to array.
						$this->reconcile_add_column(
							$column,
							$wpda_list_columns->get_column_label( $column->column_name ),
							isset( $table_column_headers[ $column->column_name ] ) ?
								$table_column_headers[ $column->column_name ] :
								$wpda_list_columns::get_default_column_label( $column->column_name )
						);
					} else {
						// Add only new columns to array.
						$column_found = false;
						foreach ( $this->wpda_table_design->listtable_column_options as $listtable_column_option ) {
							if ( isset( $column->column_name ) && isset( $listtable_column_option->column_name ) ) {
								if ( $column->column_name === $listtable_column_option->column_name ) {
									$column_found = true;
									break;
								}
							}
						}
						if ( ! $column_found ) {
							// Add column to both arrays.
							$this->reconcile_add_column(
								$column,
								$wpda_list_columns->get_column_label( $column->column_name ),
								isset( $table_column_headers[ $column->column_name ] ) ?
									$table_column_headers[ $column->column_name ] :
									$wpda_list_columns::get_default_column_label( $column->column_name )
							);
						}
					}
				}
			}
			global $wpdb;

			return
				$wpdb->update(
					$this->table_name,
					[
						'wpda_table_name'   => $this->wpda_table_name,
						'wpda_table_design' => json_encode( $this->wpda_table_design ),
					],
					[
						'wpda_table_name' => $this->wpda_table_name_original === $this->wpda_table_name ? $this->wpda_table_name : $this->wpda_table_name_original,
					]
				);
		}

		/**
		 * Reconcile a columns
		 *
		 * @param object $column Column info
		 */
		private function reconcile_add_column( $column, $label_list, $label_form ) {
			$this->wpda_table_design->listtable_column_options[] =
				[
					"column_name" => $column->column_name,
					"label"       => $label_list,
					"show"        => "on",
				];
			$this->wpda_table_design->tableform_column_options[] =
				[
					"column_name" => $column->column_name,
					"label"       => $label_form,
					"show"        => "on",
				];
		}

		/**
		 * Overwrite method get_design_table_name to (re)set base table name
		 *
		 * @return string
		 */
		public static function get_design_table_name() {
			global $wpdb;

			return $wpdb->prefix . 'wpda_project_table';
		}

		/**
		 * Get column options for specific table name and label type
		 *
		 * @param string $table_name Database table name
		 * @param string $label_type Label type
		 *
		 * @return array|null
		 */
		public static function get_column_options( $table_name, $label_type ) {
			if ( ! isset( SELF::$cache_table_options[ $table_name ] ) ) {
				global $wpdb;
				$query      = $wpdb->prepare(
					'
		              SELECT wpda_table_design
		                FROM ' . static::get_design_table_name() . '
		               WHERE wpda_table_name = %s
		            ',
					[
						$table_name,
					]
				);
				$table_json = $wpdb->get_results( $query, 'ARRAY_A' ); // WPCS: unprepared SQL OK; db call ok; no-cache ok.
				if ( 1 === $wpdb->num_rows ) {
					if ( isset( $table_json[0]['wpda_table_design'] ) ) {
						$table_obj = json_decode( $table_json[0]['wpda_table_design'] );
						if ( isset( $table_obj->table ) ) {
							SELF::$cache_table_options[ $table_name ]['table'] = $table_obj->table;
						}
						SELF::$cache_table_options[ $table_name ]['tableinfo'] = isset( $table_obj->tableinfo ) ? $table_obj->tableinfo : null;
						if ( isset( $table_obj->tableform_column_options ) ) {
							SELF::$cache_table_options[ $table_name ]['tableform'] = $table_obj->tableform_column_options;
						}
						if ( isset( $table_obj->listtable_column_options ) ) {
							SELF::$cache_table_options[ $table_name ]['listtable'] = $table_obj->listtable_column_options;
						}
						if ( isset( $table_obj->relationships ) ) {
							SELF::$cache_table_options[ $table_name ]['relationships'] = $table_obj->relationships;
						}
					}
				}
			}

			if ( 'tableform' === $label_type && isset( SELF::$cache_table_options[ $table_name ]['tableform'] ) ) {
				return SELF::$cache_table_options[ $table_name ]['tableform'];
			}
			if ( 'listtable' === $label_type && isset( SELF::$cache_table_options[ $table_name ]['listtable'] ) ) {
				return SELF::$cache_table_options[ $table_name ]['listtable'];
			}
			if ( 'relationships' === $label_type && isset( SELF::$cache_table_options[ $table_name ]['relationships'] ) ) {
				return [
					'table'         => isset( SELF::$cache_table_options[ $table_name ]['table'] ) ? SELF::$cache_table_options[ $table_name ]['table'] : null,
					'tableinfo'     => isset( SELF::$cache_table_options[ $table_name ]['tableinfo'] ) ? SELF::$cache_table_options[ $table_name ]['tableinfo'] : null,
					'relationships' => SELF::$cache_table_options[ $table_name ]['relationships'],
				];
			}
			if ( 'tableinfo' === $label_type ) {
				return isset( $table_obj->tableinfo ) ? $table_obj->tableinfo : null;
			}

			return null;
		}

		/**
		 * Overwrites method insert_reverse_engineered to add table options
		 *
		 * @param $wpda_table_name
		 * @param $wpda_table_design
		 *
		 * @return bool
		 */
		public static function insert_reverse_engineered( $wpda_table_name, $wpda_table_design ) {
			global $wpdb;

			$table_name = static::get_design_table_name();

			$wpda_table_design['table_type'] = SELF::get_table_type( $wpda_table_name );

			$wpda_table_design['listtable_column_options'] = [];
			$wpda_table_design['tableform_column_options'] = [];

			$wpda_list_columns    = WPDA_List_Columns_Cache::get_list_columns( '', $wpda_table_name );
			$table_column_headers = $wpda_list_columns->get_table_column_headers();

			foreach ( $wpda_table_design['table'] as $column ) {
				$label = $wpda_list_columns->get_column_label( $column->column_name );
				$wpda_table_design['listtable_column_options'][] =
					[
						"column_name" => $column->column_name,
						"label"       => $label,
						"show"        => "on",
					];

				$label = isset( $table_column_headers[ $column->column_name ] ) ?
					$table_column_headers[ $column->column_name ] : $wpda_list_columns::get_default_column_label( $column->column_name );
				$wpda_table_design['tableform_column_options'][] =
					[
						"column_name" => $column->column_name,
						"label"       => $label,
						"show"        => "on",
					];
			}

			return
				(
					1 === $wpdb->insert(
						$table_name,
						[
							'wpda_table_name'   => $wpda_table_name,
							'wpda_table_design' => json_encode( $wpda_table_design ),
						]
					)
				);
		}

	}

}
